@@ -26,11 +26,11 @@ Widget for the complex view ruler
 import gobject
 import gtk
 import gst
-import pitivi.instance as instance
-from zoominterface import Zoomable
-from pitivi.utils import time_to_string
+from pitivi.ui.zoominterface import Zoomable
+from pitivi.log.loggable import Loggable
+from pitivi.utils import time_to_string, Seeker
 
-class ScaleRuler(gtk.Layout, Zoomable):
+class ScaleRuler(gtk.Layout, Zoomable, Loggable):
 
     __gsignals__ = {
         "expose-event":"override",
@@ -39,15 +39,19 @@ class ScaleRuler(gtk.Layout, Zoomable):
         "button-press-event":"override",
         "button-release-event":"override",
         "motion-notify-event":"override",
+        "scroll-event" : "override",
+        "seek": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
+                [gobject.TYPE_UINT64])
         }
 
     border = 0
     min_tick_spacing = 3
 
     def __init__(self, hadj):
-        gst.log("Creating new ScaleRule")
         gtk.Layout.__init__(self)
         Zoomable.__init__(self)
+        Loggable.__init__(self)
+        self.log("Creating new ScaleRule")
         self.add_events(gtk.gdk.POINTER_MOTION_MASK |
             gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK)
         self.set_hadjustment(hadj)
@@ -64,9 +68,11 @@ class ScaleRuler(gtk.Layout, Zoomable):
 
         # position is in nanoseconds
         self.position = 0
-        self.requested_time = gst.CLOCK_TIME_NONE
-        self.currentlySeeking = False
         self.pressed = False
+        self.shaded_duration = gst.CLOCK_TIME_NONE
+        self.max_duration = gst.CLOCK_TIME_NONE
+        self.seeker = Seeker(80)
+        self.seeker.connect('seek', self._seekerSeekCb)
 
 ## Zoomable interface override
 
@@ -78,7 +84,7 @@ class ScaleRuler(gtk.Layout, Zoomable):
 ## timeline position changed method
 
     def timelinePositionChanged(self, value, unused_frame=None):
-        gst.debug("value : %r" % value)
+        self.debug("value : %r", value)
         ppos = max(self.nsToPixel(self.position) - 1, 0)
         self.position = value
         npos = max(self.nsToPixel(self.position) - 1, 0)
@@ -89,11 +95,11 @@ class ScaleRuler(gtk.Layout, Zoomable):
 ## gtk.Widget overrides
 
     def do_size_allocate(self, allocation):
-        gst.debug("ScaleRuler got %s" % list(allocation))
+        self.debug("ScaleRuler got %s", list(allocation))
         gtk.Layout.do_size_allocate(self, allocation)
-        width = max(self.getPixelWidth(), allocation.width)
-        gst.debug("Setting layout size to %d x %d"
-                  % (width, allocation.height))
+        width = max(self.getMaxDurationWidth(), allocation.width)
+        self.debug("Setting layout size to %d x %d",
+                   width, allocation.height)
         self.set_size(width, allocation.height)
         # the size has changed, therefore we want to redo our pixmap
         self.doPixmap()
@@ -104,12 +110,12 @@ class ScaleRuler(gtk.Layout, Zoomable):
         self.doPixmap()
 
     def do_expose_event(self, event):
-        gst.debug("exposing ScaleRuler %s" % list(event.area))
+        self.debug("exposing ScaleRuler %s", list(event.area))
         x, y, width, height = event.area
         if (x < self.pixmap_offset) or (x+width > self.pixmap_offset + self.pixmap_allocated_width):
-            gst.debug("exposing outside boundaries !")
+            self.debug("exposing outside boundaries !")
             self.pixmap_offset = max(0, x + (width / 2) - (self.pixmap_allocated_width / 2))
-            gst.debug("offset is now %d" % self.pixmap_offset)
+            self.debug("offset is now %d", self.pixmap_offset)
             self.doPixmap()
             width = self.pixmap_allocated_width
 
@@ -125,10 +131,9 @@ class ScaleRuler(gtk.Layout, Zoomable):
         return False
 
     def do_button_press_event(self, event):
-        gst.debug("button pressed at x:%d" % event.x)
-        if self.getDuration() <= 0:
-            gst.debug("no timeline to seek on, ignoring")
-        instance.PiTiVi.playground.switchToTimeline()
+        self.debug("button pressed at x:%d", event.x)
+        if self.getShadedDuration() <= 0:
+            self.debug("no timeline to seek on, ignoring")
         self.pressed = True
         # seek at position
         cur = self.pixelToNs(event.x)
@@ -136,48 +141,47 @@ class ScaleRuler(gtk.Layout, Zoomable):
         return True
 
     def do_button_release_event(self, event):
-        gst.debug("button released at x:%d" % event.x)
+        self.debug("button released at x:%d", event.x)
         self.pressed = False
         return False
 
     def do_motion_notify_event(self, event):
-        gst.debug("motion at event.x %d" % event.x)
+        self.debug("motion at event.x %d", event.x)
         if self.pressed:
             # seek at position
             cur = self.pixelToNs(event.x)
             self._doSeek(cur)
         return False
 
+    def do_scroll_event(self, event):
+        if event.direction == gtk.gdk.SCROLL_UP:
+            Zoomable.zoomIn()
+        elif event.direction == gtk.gdk.SCROLL_DOWN:
+            Zoomable.zoomOut()
+        # TODO: seek timeline back/forward
+        elif event.direction == gtk.gdk.SCROLL_LEFT:
+            pass
+        elif event.direction == gtk.gdk.SCROLL_RIGHT:
+            pass
+
 ## Seeking methods
 
-    def _seekTimeoutCb(self):
-        gst.debug("timeout")
-        self.currentlySeeking = False
-        if (self.requested_time != gst.CLOCK_TIME_NONE) and (self.position != self.requested_time):
-            self._doSeek(self.requested_time)
+    def _seekerSeekCb(self, seeker, position, format):
+        # clamping values within acceptable range
+        duration = self.getShadedDuration()
+        if duration in (0, gst.CLOCK_TIME_NONE):
+            return
+        if position > duration:
+            position = duration - (1 * gst.MSECOND)
+        elif position < 0:
+            position = 0
+
+        self.emit('seek', position)
+
         return False
 
-    def _doSeek(self, value, format=gst.FORMAT_TIME):
-        gst.debug("seeking to %s / currentlySeeking %r" % (gst.TIME_ARGS (value),
-                                                           self.currentlySeeking))
-        # clamping values within acceptable range
-        duration = self.getDuration()
-        if duration == gst.CLOCK_TIME_NONE:
-            return
-        if value > duration:
-            value = duration
-        elif value < 0:
-            value = 0
-        if not self.currentlySeeking:
-            self.currentlySeeking = True
-            if instance.PiTiVi.playground.seekInCurrent(value, format=format):
-                self.requested_time = gst.CLOCK_TIME_NONE
-                gobject.timeout_add(80, self._seekTimeoutCb)
-                self.timelinePositionChanged(value)
-            else:
-                self.currentlySeeking = False
-        elif format == gst.FORMAT_TIME:
-            self.requested_time = value
+    def _doSeek(self, value, format=gst.FORMAT_TIME, on_idle=False):
+        self.seeker.seek(value, format, on_idle)
 
 ## Drawing methods
 
@@ -216,18 +220,38 @@ class ScaleRuler(gtk.Layout, Zoomable):
 
     def draw(self, context):
         rect = self.get_allocation()
-        gst.debug("Ruler draw %s" % list(rect))
+        self.debug("Ruler draw %s", list(rect))
         self.drawBackground(context, rect)
         self.drawRuler(context, rect)
 
-    def getDuration(self):
-        if instance.PiTiVi.current:
-            return instance.PiTiVi.current.timeline.getDuration()
-        else:
-            return gst.CLOCK_TIME_NONE
+    def setShadedDuration(self, duration):
+        self.info("start/duration changed")
+        self.queue_resize()
 
-    def getPixelWidth(self):
-        return self.nsToPixel(self.getDuration())
+        self.shaded_duration = duration
+
+        if duration < self.position:
+            position = duration - gst.NSECOND
+        else:
+            position = self.position
+
+        self._doSeek(position, gst.FORMAT_TIME, on_idle=True)
+
+    def getShadedDuration(self):
+        return self.shaded_duration
+
+    def getShadedDurationWidth(self):
+        return self.nsToPixel(self.getShadedDuration())
+
+    def setMaxDuration(self, duration):
+        self.queue_resize()
+        self.max_duration = duration
+
+    def getMaxDuration(self):
+        return self.max_duration
+
+    def getMaxDurationWidth(self):
+        return self.nsToPixel(self.getMaxDuration())
 
     def getPixelPosition(self):
         return 0
@@ -240,18 +264,14 @@ class ScaleRuler(gtk.Layout, Zoomable):
         context.fill()
         context.stroke()
 
-        if self.getDuration() > 0:
+        if self.getShadedDuration() > 0:
             context.set_source_rgb(0.8, 0.8, 0.8)
-            context.rectangle(0, 0, self.getPixelWidth(), allocation.height)
+            context.rectangle(0, 0, self.getShadedDurationWidth(), allocation.height)
             context.fill()
             context.stroke()
 
         context.restore()
 
-    def startDurationChanged(self):
-        gst.info("start/duration changed")
-        self.queue_resize()
-
     def drawRuler(self, context, allocation):
         # there are 4 lengths of tick mark:
         # full height: largest increments, 1 minute
@@ -325,7 +345,7 @@ class ScaleRuler(gtk.Layout, Zoomable):
         context.restore()
 
     def drawPosition(self, context, allocation):
-        if self.getDuration() <= 0:
+        if self.getShadedDuration() <= 0:
             return
         # a simple RED line will do for now
         xpos = self.nsToPixel(self.position) + self.border
